/*
 * Interprocess communication abstractions.
 *
 * Copyright (C) 2018, 2019, 2021, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/re/c/dataspace.h>
#include <l4/sys/ipc.h>
#include <l4/sys/utcb.h>
#include <l4/sys/types.h>

#include <sys/types.h>



EXTERN_C_BEGIN

/* Supporting operations. */

long _expect_capabilities(l4_buf_regs_t *bregs, int number);
long _expect_capability(l4_buf_regs_t *bregs, int item);
long _expect_fpage(l4_buf_regs_t *bregs, int item, l4_umword_t map_control, l4_fpage_t fpage);
void _export_capability(l4_msgtag_t tag, l4_msg_regs_t *mregs, int item, l4_cap_idx_t ref);
void _export_fpage(l4_msgtag_t tag, l4_msg_regs_t *mregs, int item, l4_snd_fpage_t fpage);
void _export_page(l4_msgtag_t tag, l4_msg_regs_t *mregs, int item, l4_umword_t hot_spot, l4_fpage_t fpage);
void _free_expected_capabilities(l4_buf_regs_t *bregs, int number);
long _import_capability(l4_msgtag_t tag, l4_buf_regs_t *bregs, l4_msg_regs_t *mregs, int item, l4_cap_idx_t *ref);
long _import_dataspace(l4_msgtag_t tag, l4_buf_regs_t *bregs, l4_msg_regs_t *mregs, int item, l4re_ds_t *mem, l4_addr_t *addr);
long _import_fpage(l4_msgtag_t tag, l4_buf_regs_t *bregs, l4_msg_regs_t *mregs, int item, l4_snd_fpage_t *fpage);

/* Convenience operations. */

l4_msgtag_t util_ipc_reply(l4_msgtag_t tag);
l4_msgtag_t util_ipc_request(l4_msgtag_t tag, l4_cap_idx_t endpoint);
l4_msgtag_t util_ipc_send(l4_msgtag_t tag, l4_cap_idx_t endpoint);

EXTERN_C_END
